
#include <stdio.h>
#include "mex.h"
#include <math.h>

#undef		MIN
#undef		MAX
#define		MIN(a, b)       ((a) < (b) ? (a) : (b))	
#define		MAX(a, b)       ((a) > (b) ? (a) : (b))	

inline int myfloor(double d) {
    int f = d;
    return f;
}

void main_stuff(double *gpad, double *w, int Mp, int Np, int win, int jj, double *kernel, double h, int SearchWin)
{
    int M, N;
    int j, i;
    int jc, ic;
    int jr, ir; 
    int winLength = 2*win+1;
    double sse;
    double block_ref, block_cur, block_err, alpha;
    int jc_min, jc_max, ic_min, ic_max;

    M    = Mp - 2*win;
    N    = Np - 2*win;

    jj = jj-1;
    ir = jj % M;
    jr = myfloor(jj/M);
    
    jc_min = MAX(0,jr-SearchWin);
    jc_max = MIN(N,jr+SearchWin);
    ic_min = MAX(0,ir-SearchWin);
    ic_max = MIN(M,ir+SearchWin);

    for (j=0; j<N; j++)
        for (i=0; i<M; i++)
        {
        w[(j*M)+i] = 0;
        }
    
    for (jc=jc_min; jc<jc_max; jc++)
        for (ic=ic_min; ic<ic_max; ic++)
        {
        sse = 0.0;
        
        if (win>0)
        {
        for (j=0; j<winLength; j++)
            for (i=0; i<winLength; i++)
            {
            block_ref = gpad[(jr+j)*Mp+ir+i];
            block_cur = gpad[(jc+j)*Mp+ic+i];
            block_err = block_ref-block_cur;
            alpha     = kernel[j*winLength+i];
            sse       = sse + block_err*block_err*alpha;
            }
        }
        else
        {
            block_ref = gpad[(jr)*Mp+ir];
            block_cur = gpad[(jc)*Mp+ic];
            block_err = block_ref-block_cur;
            sse       = sse + block_err*block_err;
        }
        
        w[(jc*M)+ic] = exp(-sse/(h*h));
        }
}



/////////////////////////
// Interface Function
/////////////////////////
void mexFunction(int nlhs, mxArray *plhs[],
        int nrhs, const mxArray *prhs[])
{
    double *gpad, *w, *kernel;
    int jj, win, SearchWin;
    int Mp, Np, M, N;
    double h;
    
    /////////////////////
    // Inputs
    /////////////////////
    // Input 1
    gpad = mxGetPr(prhs[0]);
    Mp   = mxGetM(prhs[0]);
    Np   = mxGetN(prhs[0]);
    
    // Input 2
    jj  = mxGetScalar(prhs[1]);
    
    // Input 3
    win  = mxGetScalar(prhs[2]);
    
    // Input 3
    kernel = mxGetPr(prhs[3]);
    
    // Input 4
    h = mxGetScalar(prhs[4]);
    
    // Input 5
    SearchWin = mxGetScalar(prhs[5]);
    
    // Calculate M and N
    M    = Mp - 2*win;
    N    = Np - 2*win;
    
    ////////////////////
    // Outputs
    ////////////////////
    plhs[0] = mxCreateDoubleMatrix(M, N, mxREAL);
    w = mxGetPr(plhs[0]);
    
    main_stuff(gpad,w,Mp,Np,win,jj,kernel,h,SearchWin);
}