function [zhat ActualRatio] = mcnlm(y,hr,hs,win,SearchWin,SamplingRatio,method)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% zhat = mcnlm(y,hr,hs,win,SearchWin,SamplingRatio,method)
% denoises an image y using Monte-Carlo Non-local Means
%
% Input: y  - noisy image, size of MxN
%        hr - range parameter
%        hs - spatial parameter
%        win - radius of patch size
%        SearchWin - radius of search neighborhood
%        Sampling ratio - a number in [0,1]
%        method   - one of the following options:
%                   'oracle','uniform','spatial','intensity'
% 
% Output: zhat - denoised image
%         ActualRatio - actual sampling ratio
% 
% Stanley Chan
% Harvard University
% Jul 23, 2013
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

npca     = 10;
ypca     = pca_projection(y,win,npca);
n        = numel(y);
ymean    = reshape(mean(ypca,1),size(y));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Main Denoising Step
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% tic
switch method
    case 'uniform'
            [zhat ActualRatio] = mcnlm_uniform_sampling(y,ymean,ypca,hr,hs,SearchWin,SamplingRatio,1:n);

    case 'spatial'
            [zhat ActualRatio] = mcnlm_spatial_sampling(y,ymean,ypca,hr,hs,SearchWin,SamplingRatio,1:n);

    case 'intensity'
            [zhat ActualRatio] = mcnlm_intensity_sampling(y,ymean,ypca,hr,hs,SearchWin,SamplingRatio,1:n);
            
    case 'oracle'
            [zhat ActualRatio] = mcnlm_oracle_sampling(y,ymean,ypca,hr,hs,SearchWin,SamplingRatio,1:n);
 
    otherwise
        error('Please input the correct method. \n');
end
% toc


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reshape Output
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
zhat = reshape(zhat,size(y));

