clear all
close all
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Demo file of Monte Carlo Non-Local Means
%
% This demo file illustrates the basic usage of MCNLM
% for single image denoising.
%
% Stanley Chan
% Copyright 2013 Harvard University
% Last update: Nov 20, 2013
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

addpath(genpath('./code/'));
addpath(genpath('./utilities/'));

% Load Data
folder   = './data/standard_images/';
filename = 'hill.png';
imgname  = sprintf('%s%s',folder,filename);
z0       = im2double(imread(imgname)); % z0 = clean image

% Make sure the image is gray scale.
if size(z0,3)==3 
    z0 = rgb2gray(z0);
end

% Parameters
sigma         = 15; % Noise standard deviation, in scale [0,255]
SearchWin     = 10; % Spatial search window radius. E.g., for 21x21, SearchWin = 10
hs            = SearchWin/3; % Spatial parameter
hr            = sigma/255;   % Intensity parameter
win           = 2;           % Patch radius. E.g., for 5x5, win = 2
SamplingRatio = 0.2;         % Sampling Ratio, in scale [0,1]
method        = 'spatial';   % Method - 'spatial' for spatial approx sampling
                             %        - 'intensity' for intensity approx sampling
                             %        - 'oracle' for oracle sampling
                             %        - 'uniform' for uniform sampling
                             
% Construct a noisy image
y = z0 + random('normal',0, sigma/255, size(z0));
y(y>1) = 1;
y(y<0) = 0;

% Classical Denoise
z_full = mcnlm(y,hr,hs,win,SearchWin,1,method);

% MCNLM Denoise
z_mcnlm = mcnlm(y,hr,hs,win,SearchWin,SamplingRatio,method);

% Output and display results
figure(1);
subplot(221);
imshow(z0);
title('Ground Truth');

subplot(222);
imshow(y);
FigTitle1 = sprintf('Noisy %3.2f dB',psnr(y,z0));
title(FigTitle1);

subplot(223);
imshow(z_full);
FigTitle3 = sprintf('Classical %3.2f dB, 100%%', psnr(z_full,z0));
title(FigTitle3);

subplot(224);
imshow(z_mcnlm);
FigTitle4 = sprintf('MCNLM %3.2f dB, 20%%', psnr(z_mcnlm,z0));
title(FigTitle4);
