clear all
close all
clc

% This routine computes the numerical values for the example after Thm 1
% Purpose: illustrate Theorem 1 by comparing the theoretical bound
% and the experimental histogram
% 
% Stanley Chan @ Harvard
% May 23, 2013
% Aug 16, 2013

addpath(genpath('../code/'));

% Signal
n = 10000;
x = linspace(0,1,round(n/4));
f = [0.25*(sin(5.5*x)+1.5) 0.5*ones(1,round(n/4)) 0.25*(cos(2.4*x)+1) 0.25*ones(1,round(n/4))];
f = f';

g = f + random('normal',0,5/255,size(f));
g(g>1) = 1;
g(g<0) = 0;


% Denoising Parameters
h         = 15/255;
win       = 2;
kernel    = make_kernel_pyramid(win,1);
SearchWin = n;
gpad      = padarray(g,[win,0],'symmetric');

% Sampling Probability
p = 0.05*ones(n,1);

% Data
w = compute_weight_1D(gpad, 5001, win, kernel, h, SearchWin);
x = g(:);
w = w(:);

% Statistics of Data
mu_A  = sum(w.*x)/n;
mu_B  = sum(w)/n;
mu    = mu_A/mu_B;

% Main Experiment
t   = 0.01; 
a1  = abs(w.*x-mu*w-w*t);
a2  = abs(w.*x-mu*w+w*t);
M1  = max(abs(a1).*(max(1,(1-p)./p)));
M2  = max(abs(a2).*(max(1,(1-p)./p)));

d1 = (1/n)*sum( a1.^2.*(1-p)./p ) + mu_B*M1*t/3;
d2 = (1/n)*sum( a2.^2.*(1-p)./p ) + mu_B*M2*t/3;

P_theo = exp(-(n/2)*(t^2)*(mu_B^2/ d1) ) + exp(-(n/2)*(t^2)*(mu_B^2/ d2) ) + exp(-sum(p));

SumAlpha2 = sum(a1.^2)/n;
SumBeta2  = sum(a2.^2)/n;

fprintf('sum(alpha^2)/n = %3.5e \n', SumAlpha2);
fprintf('sum(beta^2)/n  = %3.5e \n', SumBeta2);
fprintf('mu_B           = %3.5e \n', mu_B);
fprintf('M1             = %3.5e \n', M1);
fprintf('M2             = %3.5e \n', M2);
fprintf('Bound          = %3.5e \n', P_theo);
fprintf('Remark: Randomness in number is caused by noise added to data.\n');
