%%%%%%%%%%%%%%%%%%%%%
% Generate Data for Fig.2-Fig.3
% 
% Warning: This program takes 10+ hours
%          if set hs = 1e16 and SearchWin = n
% 
% Stanley Chan
% Harvard University
% Jul 23, 2013
% Aug 16, 2013
%%%%%%%%%%%%%%%%%%%%%

clear all
close all
clc

addpath(genpath('../offshelf_packages/'));
addpath(genpath('../utilities/'));
addpath(genpath('../code/'));
addpath(genpath('../data/'));


currentFolder = pwd;

% Create clean data
z = im2double(imread('../data/Fig3_data/building_16.png'));

% Create noisy data
sigma     = 15;
y         = z + random('normal',0,sigma/255,size(z));
y(y>1)    = 1;
y(y<0)    = 0;

% Denoising Parameters
win       = 2;                      % patch radius
SearchWin = 20;                     % radius of spatial search neigborhood
hr        = sigma/255;              % range parameter
hs        = SearchWin/3;            % spatial parameter
SamplingRatio = 0.1;                % sampling ratio

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MCNLM
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
zhat = mcnlm_nThreads(y,hr,hs,win,SearchWin,SamplingRatio,4,'uniform');
imshow(zhat);
PSNR = psnr(z,zhat);
T = sprintf('PSNR = %3.2f dB', PSNR);
title(T);

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% BM3D 
% modified Feb 28, 2014
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[~,zhat_bm3d] = BM3D(1, y, sigma);
PSNR_bm3d     = psnr(zhat_bm3d,z);
% save('../data/Fig3_data/psnr_data_bm3d','PSNR_bm3d');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% KD Tree
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cd ../offshelf_packages/;
imwrite(y,'noisy.png');
npca = 10;
patch_diameter = 2*win+1;
s = (patch_diameter - 1)/6;
full_cmd = sprintf('ImageStack.exe -load noisy.png -nlmeans %3.4g %3.4g %3.4g %3.4g exact -save output.png',s,npca,hs,hr*sqrt(npca));
system(full_cmd);
zhat_kd = im2double(imread('output.png'));
cd(currentFolder);
PSNR_kd = psnr(zhat_kd,z);
% save('../data/Fig3_data/psnr_data_kd','PSNR_kd');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Adaptive Manifold
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
hr = 2*sigma/255;
[~,zhat_am]     = adaptive_manifold_filter(y, hs, hr);
zhat_am         = zhat_am(:,:,1);
PSNR_am         = psnr(zhat_am,z);
% save('../data/Fig3_data/psnr_data_am','PSNR_am');