clear all
close all
clc

% This MATLAB routine plots Fig.4
% Pr[ Sn - E[Sn] > epsilon ] as a function of epsilon
% Purpose: to illustrate lemma 1
%
% Stanley Chan @ Harvard
% May 23, 2013
% Aug 16, 2013


%=== Toggle this part to execute the experiment ===%
% p           = 0.5;
% e_set       = logspace(-3,-1,20);
% L           = length(e_set);
% m           = 1e4;      % We set m = 1e6 in the paper
% n           = 1e3;
% 
% Err         = zeros(L,1);
% for i=1:L
%     fprintf('i = %3g \n', i);
%     e = e_set(i);
%     S = binornd(n,p,[m,1])/n;
%     Err(i) = nnz(S-p>e);
% end


%=== Display the processed data ===%
load('../data/Fig4_data');

fontsize = 12;
set(0,'defaultaxesfontsize',fontsize);
set(0,'defaulttextfontsize',fontsize);

fontname = 'Times New Roman';
set(0,'defaultaxesfontname',fontname);
set(0,'defaulttextfontname',fontname);

fontweight = 'normal';
set(0,'defaultaxesfontweight',fontweight);
set(0,'defaulttextfontweight',fontweight);

figure;
semilogy(e_set, Err/m, 'kx', 'LineWidth', 2); hold on; 
semilogy(e_set, (p*(1-p)./(n*e_set.^2)), 'b-^', 'LineWidth', 2); 
semilogy(e_set, ( exp(-n*e_set.^2./(2*(p*(1-p) + e_set/6))) ), 'r-o', 'LineWidth', 2);
axis([0 0.1 1e-8 1e4]);
grid on;
xlabel('$\varepsilon$', 'interpreter', 'latex', 'fontsize', 14);
title('$\Pr[ S_n - E[S_n] > \varepsilon]$', 'interpreter', 'latex', 'fontsize', 14);
legend('True probabilities', 'Law of large numbers bounds', 'Concentration of measure bounds');
% legend('True probabilities', 'Law of large numbers bounds', 'Large deviations bounds');
export_fig chan04.eps -transparent -RGB;