clear all
close all
clc

% This routine plots Fig.6(b)-Fig.6(e)
% Purpose: compares different sampling patterns
% 
% Stanley Chan @ Harvard
% Aug 16, 2013
% Nov 14, 2013

addpath(genpath('../code/'));
addpath(genpath('../utilities/'));

f = im2double(imread('cameraman.tif'));
n = numel(f);

g = f + random('normal',0,5/255,size(f));
g(g>1) = 1;
g(g<0) = 0;

% Denoising Parameters
h         = 15/255;
win       = 2;
kernel    = make_kernel_pyramid(win, 2);
SearchWin = n;
gpad      = padarray(g,[win,win],'symmetric');

% Data
wr = compute_weight_2D(gpad, sub2ind([256,256],75,75), win, kernel, h, SearchWin);
H  = fspecial('gaussian',[501,501],50);
ws = H(250+[-73:256-74],250+[-73:256-74]);
ws = ws/max(ws(:));

w = wr(:).*ws(:); % full NLM weight
x = g(:);

mu_A  = sum(w.*x)/n;
mu_B  = sum(w)/n;
mu    = mu_A/mu_B;

xi = 0.05;
k  = n*xi;

% Oracle w
b = w;
t = max( (1/k)*sum(abs(b(:))), max(abs(b)) ) + sqrt(eps);
myfun    = @(v)objfun(v,t,b,k);
tau      = fzero(myfun,[0,1e16]);
p_oracle = max(abs(b)/t, min(1,abs(b)*tau));

% spatial bs
b = ws(:);
t = max((1/k)*sum(abs(b(:))),max(abs(b))) + sqrt(eps);
myfun    = @(v)objfun(v,t,b,k);
tau       = fzero(myfun,[0,1e16]);
p_spatial = max(abs(b)/t, min(1,abs(b)*tau));

% intensity bi
gmean = imfilter(g, ones(2*win+1)/(2*win+1)^2, 'symmetric');
pmean = g(75+[-win:win], 75+[-win:win]);
pmean = mean(pmean(:));
b = exp(-(2*win+1)*(gmean-pmean).^2/(2*h^2));
b = b(:);
t = max((1/k)*sum(abs(b(:))),max(abs(b))) + sqrt(eps);
myfun    = @(v)objfun(v,t,b,k);
tau      = fzero(myfun,[0,1e16]);
p_intensity = max(abs(b)/t, min(1,abs(b)*tau));

% spatial + intensity bi * bs
gmean = imfilter(g, ones(2*win+1)/(2*win+1)^2, 'symmetric');
pmean = g(75+[-win:win], 75+[-win:win]);
pmean = mean(pmean(:));
bI = exp(-(2*win+1)*(gmean-pmean).^2/(2*h^2));
bI = bI(:);
bS = ws(:);
b  = bI.*bS;
t = max((1/k)*sum(abs(b(:))),max(abs(b))) + sqrt(eps);
myfun    = @(v)objfun(v,t,b,k);
tau       = fzero(myfun,[0,1e16]);
p_both    = max(abs(b)/t, min(1,abs(b)*tau));

% Plot Result
Fig = figure(1);
imagesc(reshape(p_oracle,256,256));
colorbar;
axis tight
Fig_prop = get(gcf);
boundbox = Fig_prop.PaperPosition;
width    = boundbox(3);
height   = boundbox(4);
set(gcf, 'PaperPosition', [0, 0, width, height]);
set(gcf, 'PaperSize', [width height]);
export_fig chan06b.eps -transparent -RGB;
% saveas(Fig, 'sampling_oracle.eps','epsc');

Fig = figure(2);
imagesc(reshape(p_spatial,256,256));
colorbar;
axis tight
Fig_prop = get(gcf);
boundbox = Fig_prop.PaperPosition;
width    = boundbox(3);
height   = boundbox(4);
set(gcf, 'PaperPosition', [0, 0, width, height]);
set(gcf, 'PaperSize', [width height]);
export_fig chan06c.eps -transparent -RGB;
% saveas(Fig, 'sampling_spatial.eps','epsc');

Fig = figure(3);
imagesc(reshape(p_intensity,256,256));
colorbar;
axis tight
Fig_prop = get(gcf);
boundbox = Fig_prop.PaperPosition;
width    = boundbox(3);
height   = boundbox(4);
set(gcf, 'PaperPosition', [0, 0, width, height]);
set(gcf, 'PaperSize', [width height]);
export_fig chan06d.eps -transparent -RGB;
% saveas(Fig, 'sampling_intensity.eps','epsc');

Fig = figure(4);
imagesc(reshape(p_both,256,256));
colorbar;
axis tight
Fig_prop = get(gcf);
boundbox = Fig_prop.PaperPosition;
width    = boundbox(3);
height   = boundbox(4);
set(gcf, 'PaperPosition', [0, 0, width, height]);
set(gcf, 'PaperSize', [width height]);
export_fig chan06e.eps -transparent -RGB;
% saveas(Fig, 'sampling_both.eps','epsc');
