function [psnr1,psnr2,SamplingRatio_list,runtime1,runtime2,zden1,zden2,znum1,znum2,MeanActualSamplingRatio1,MeanActualSamplingRatio2] = core_outer(batchid)

% Load database information
load('../../data/Fig8_data/data.mat');         % noisy patch
load('../../data/Fig8_data/fname_list.mat');   % image file name
ImageNames = fnames';

% Prepare noisy and ground truth patches
win    = 2;
h      = 0.0729;
z_full = reshape(xL(5+[-win:win],5+[-win:win],:),(2*win+1)^2,2000);
z      = z_full(((2*win+1)^2+1)/2,:)';
y      = yL(5+[-win:win],5+[-win:win],:);
ymean  = mean(reshape(y,(2*win+1)^2,2000));
clear xL yL ex1;

% Parameters
iset  = 1:2000;
m      = length(iset);
d      = (2*win+1)^2;

% Sampling Ratio: 1e-6 to 1e-2
SamplingRatio_list = logspace(-6,-2,20);
SamplingRatio_length = length(SamplingRatio_list);

% Image ID: Each batch contains 1000 images
fid_list   = (batchid-1)*1000+(1:1000);
fid_length = length(fid_list);

% Initialization
znum1 = zeros(m,SamplingRatio_length,fid_length);
znum2 = zeros(m,SamplingRatio_length,fid_length);

zden1 = zeros(m,SamplingRatio_length,fid_length);
zden2 = zeros(m,SamplingRatio_length,fid_length);

ActualSamplingRatio1 = zeros(SamplingRatio_length,fid_length);
ActualSamplingRatio2 = zeros(SamplingRatio_length,fid_length);

timer1 = zeros(SamplingRatio_length,fid_length);
timer2 = zeros(SamplingRatio_length,fid_length);

Qlevels = 8;

parfor fid_ptr = 1:fid_length           % if using for demo, set fid_length = 10
    fprintf('fid ptr = %3g \n', fid_ptr);
    
    %==============  Load Data =============%
    fid    = fid_list(fid_ptr);
    fname  = ImageNames{fid};

    x_full = im2double(imread(fname));
    if size(x_full,3)>1
        x_full = rgb2gray(x_full);
    end
    x      = padarray(x_full, [win, win], 'symmetric');
    n      = numel(x_full);
    %========= END Load Data   =============%
    
    
    

    %===== Build Statistics of Database =====%
    %---     assume performed offline     ---%
    Xstat               = compute_xstat( x_full, win, Qlevels );
    xmean               = Xstat.xmean;
    xmeansort           = Xstat.xmeansort;
    xhist_boundary      = Xstat.xhist_boundary;
    xhist_count         = Xstat.xhist_count;
    xhist_idx_randperm  = Xstat.xhist_idx_randperm;
    %===== END Build Statistics of Database =====%
    
    
    
    
    %====== Main Procedure ====%
    %--- online computation ---%
    [znum1tmp,znum2tmp,zden1tmp,zden2tmp,ActualSamplingRatio1tmp,ActualSamplingRatio2tmp,timer1tmp,timer2tmp] = core_inner(y,ymean,x,xmean,xmeansort,h,win,d,m,n,xhist_idx_randperm,xhist_count,xhist_boundary,iset,SamplingRatio_list,SamplingRatio_length);
    znum1(:,:,fid_ptr) = znum1tmp;
    znum2(:,:,fid_ptr) = znum2tmp;
    zden1(:,:,fid_ptr) = zden1tmp;
    zden2(:,:,fid_ptr) = zden2tmp;
    ActualSamplingRatio1(:,fid_ptr) = ActualSamplingRatio1tmp;
    ActualSamplingRatio2(:,fid_ptr) = ActualSamplingRatio2tmp;
    timer1(:,fid_ptr) = timer1tmp;
    timer2(:,fid_ptr) = timer2tmp;


    %=== END Main Procedure ===%

end

psnr1 = zeros(SamplingRatio_length,1);
psnr2 = zeros(SamplingRatio_length,1);
runtime1 = zeros(SamplingRatio_length,1);
runtime2 = zeros(SamplingRatio_length,1);

zhat1 = sum(znum1,3)./sum(zden1,3);
zhat2 = sum(znum2,3)./sum(zden2,3);
    
for sid=1:SamplingRatio_length
    psnr1(sid) = -10*log10(mean( (z-zhat1(:,sid)).^2 ));
    psnr2(sid) = -10*log10(mean( (z-zhat2(:,sid)).^2 ));
    
    runtime1(sid) = mean(timer1(sid,:));
    runtime2(sid) = mean(timer2(sid,:));
end

zden1 = sum(zden1,3);
zden2 = sum(zden2,3);

znum1 = sum(znum1,3);
znum2 = sum(znum2,3);

MeanActualSamplingRatio1 = mean(ActualSamplingRatio1,2)';
MeanActualSamplingRatio2 = mean(ActualSamplingRatio2,2)';

