/* Monte Carlo Non-Local Means
 * 
 */

#include <stdio.h>
#include "mex.h"
#include <math.h>
#include "matrix.h"
#include <time.h>

#undef		MIN
#undef		MAX
#define		MIN(a, b)       ((a) < (b) ? (a) : (b))	
#define		MAX(a, b)       ((a) > (b) ? (a) : (b))

//         ///////////////////////////////////////
//         // -- Print Screen to Monitor --     //
//         if(i%10==0)
//         {
//             printf("Doing %3d / %3d \n", i, m);
//             mexEvalString("drawnow;");
//         }
//         // -- End Print Screen to Monitor -- //
//         ///////////////////////////////////////


//     main_stuff(znum,zden,ActualNumSample,y,ymean,x,xmean,h,win,tau,
//             xhist_idx_randperm,xhist_idx_randperm_length,xhist_count,
//             wmean,wmeanpos,
//             Qlevels,SamplingRatio,
//             d,n,m,Mp,Np,iset);

void main_stuff(double *znum, double *zden, double *ActualNumSample,
                double *y, double *ymean, double *x, double *xmean,
                double h, int win, double *tau,
                double *xhist_idx_randperm, int xhist_idx_randperm_length, double *xhist_count, double *xhist_boundary,
                int Qlevels, double SamplingRatio,
                int d, int n, int m, int Mp, int Np, 
                double *iset)
{
//     double* IID = NULL;
//     IID = new double[n];
//     for (j=0; j<n; j++)
//     {
//         IID[j] = ((double)rand()/RAND_MAX);
//     }
    srand (time(NULL));
    
    int M = Mp - 2*win;
    int N = Np - 2*win;
    int winLength = 2*win+1;
    
    double invh2 = 1/(2*h*h);
    double sse, res, w;
    int i,j, i1, j1, i2, j2;
    double num, den;

    double NumSample;
    double p;

    double* wmean1 = NULL;
    wmean1 = new double[Qlevels];
    int wmaxpos;
    double wmax;
    double wmean2;
    
    double Stage1_prob;
    int Stage1_Nsample;
    
    for (int ii=0;ii<m;ii++)
    {
   
        NumSample = 0;
        num = 0.0;
        den = 0.0;
        
        i = iset[ii]-1;

        //--- Compute wmean for Qlevel bins  ---//
        wmax = 0;
        for (int qidx=0; qidx<Qlevels; qidx++)
        {
            wmean1[qidx] = exp(- (ymean[i]-xhist_boundary[qidx])*(ymean[i]-xhist_boundary[qidx])*invh2 );
            if (wmean1[qidx]>=wmax)
            {
                wmax = wmean1[qidx];
                wmaxpos = qidx;
            }
        }
        wmean1[wmaxpos] = 1;
        for (int qidx=Qlevels; qidx>wmaxpos; qidx--)
        {
            wmean1[qidx] = wmean1[qidx-1];
        }

        //--- Sampling for Qlevels bins ---//
        for (int qidx=0; qidx<Qlevels; qidx++)
        {

            Stage1_prob    = wmean1[qidx]*tau[i];
            Stage1_Nsample = (int) ( MIN( (Stage1_prob*xhist_count[qidx]+0.5), xhist_count[qidx] ) );
            
            if (Stage1_Nsample>0)
            {
                for (int jj=0; jj<Stage1_Nsample; jj++)
                {
                    j  = xhist_idx_randperm[ xhist_idx_randperm_length*qidx + jj]-1;
                    i1 = j % M;
                    j1 = (int)(j/M);
                    
                    wmean2 = exp(-(xmean[j]-ymean[i])*(xmean[j]-ymean[i])*invh2);
                    p = wmean2*tau[i]/Stage1_prob;
                    if ( (double)rand() < p*RAND_MAX )
                    {
                        sse = 0.0;
                        for (j2=0;j2<winLength;j2++)
                        for (i2=0;i2<winLength;i2++)
                        {
                            res = y[i*winLength*winLength+j2*winLength+i2]-x[(j1+j2)*Mp+(i1+i2)];
                            sse = sse + res*res;
                        }
                        w = exp(-sse*invh2);
                        
                        num = num + w*x[(j1+win)*Mp+(i1+win)]/p;
                        den = den + w/p;
                        NumSample = NumSample + 1;
                    }
                }
            }
        }

        znum[ii] = num;
        zden[ii] = den;
        ActualNumSample[ii] = NumSample;
    }
//     delete[] IID;
}  


                
                

/////////////////////////
// Interface Function
/////////////////////////
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{

    double *znum;   // znum  is the denoised output
    double *zden;   // zden  is the denoised output
    double *ActualNumSample;
    
    double *y, *ymean;      // y     is noisy input
    double *x, *xmean;      // x     is database input
    
    int win;
    double h;
    double *tau;
    double SamplingRatio;
    
    double *xhist_idx_randperm, *xhist_count, *xhist_boundary;
    int xhist_idx_randperm_length, Qlevels;
    
    double *iset;  // a vector listing pixels to be denoised

    int d, n, m;    // n is number of pixels in the database
    int Mp, Np;

    const mwSize *xSize;
    mwSize xDims;
    
    const mwSize *ySize;
    mwSize yDims;

    /////////////////////
    // Inputs
    // nlm_external_mean(y,ymean,x,xmean,h,win,tau,Stage1_idx,Stage1_numel,Stage1_prob,iset);
    // (ymean,h,SamplingRatio,tau,Qlevels,m,n,xhist_boundary,xhist_count,xhist_idx_randperm)
    /////////////////////
    // Input 0 
    y     = mxGetPr(prhs[0]);
    yDims = mxGetNumberOfDimensions(prhs[0]);
    ySize = mxGetDimensions(prhs[0]);
    
    // Input 1 
    ymean     = mxGetPr(prhs[1]);

    // Input 2 
    x     = mxGetPr(prhs[2]);
    xDims = mxGetNumberOfDimensions(prhs[2]);
    xSize = mxGetDimensions(prhs[2]);

    // Input 3 
    xmean     = mxGetPr(prhs[3]);

    // Input 4
    h = mxGetScalar(prhs[4]);
    if (h<=0)
        mexErrMsgTxt("The parameter hr must be positive. Please retry. \n");
    
    // Input 5 radius of patch size
    win = mxGetScalar(prhs[5]);
    
    // Input 6 sampling parameter
    tau = mxGetPr(prhs[6]);  
    
    // Input 7 indices for each bin (randomly permuted)
    xhist_idx_randperm        = mxGetPr(prhs[7]);
    xhist_idx_randperm_length = mxGetM(prhs[7]);
    
    // Input 8  number of indices for each bin
    xhist_count      = mxGetPr(prhs[8]);
    Qlevels          = MAX(mxGetM(prhs[8]),mxGetN(prhs[8]));
            
    // Input 9 boundary of bins
    xhist_boundary   = mxGetPr(prhs[9]);
    
    // Input 10 working set
    iset = mxGetPr(prhs[10]);
    m    = MAX(mxGetM(prhs[10]), mxGetN(prhs[10]));

    // Input 11 sampling ratio
    SamplingRatio = mxGetScalar(prhs[11]);
    
    
    // Calculate Output Dimensions
    int zDims;
    int zSize[2];

    Mp = xSize[0];
    Np = xSize[1];
    
    zDims    = 2;
    zSize[0] = m;
    zSize[1] = 1;
    d        = ySize[0]*ySize[1];
    n        = (Mp-2*win)*(Np-2*win);
    
    
    ////////////////////
    // Outputs
    ////////////////////
    plhs[0]     = mxCreateNumericArray(zDims, zSize, mxDOUBLE_CLASS, mxREAL);
    znum        = (double *)mxGetData(plhs[0]);  

    plhs[1]     = mxCreateNumericArray(zDims, zSize, mxDOUBLE_CLASS, mxREAL);
    zden        = (double *)mxGetData(plhs[1]);  

    plhs[2]      = mxCreateNumericArray(zDims, zSize, mxDOUBLE_CLASS, mxREAL);
    ActualNumSample = (double *)mxGetData(plhs[2]);  
    
    main_stuff(znum,zden,ActualNumSample,y,ymean,x,xmean,h,win,tau,
            xhist_idx_randperm,xhist_idx_randperm_length,xhist_count,xhist_boundary,
            Qlevels,SamplingRatio,
            d,n,m,Mp,Np,iset);
}

