/* Monte Carlo Non-Local Means
 * 
 */

#include <stdio.h>
#include "mex.h"
#include <math.h>
#include "matrix.h"
#include <time.h>

#undef		MIN
#undef		MAX
#define		MIN(a, b)       ((a) < (b) ? (a) : (b))	
#define		MAX(a, b)       ((a) > (b) ? (a) : (b))

// znum,zden,ActualNumSample,y,x,h,win,SamplingRatio,d,n,m,Mp,Np
void main_stuff(double *znum, double *zden, double *ActualNumSample,
                double *y, double *x,
                double h, int win, double SamplingRatio, 
                int d, int n, int m, 
                int Mp, int Np, double *iset)
{
    srand (time(NULL));
    
    double invh2 = 1/(2*h*h);
    double sse, w, res;
    int i,j; 
    int i1, j1, i2, j2;
    double num, den;
    int winLength = 2*win+1;
    int M = Mp-2*win;

    int* PickedSampleTmp = NULL;
    PickedSampleTmp = new int[n];
    int counter = 0;
    for (j=0; j<n; j++)
    {
        if ((double)rand()/RAND_MAX <= SamplingRatio)
        {
            PickedSampleTmp[counter] = j;
            counter = counter + 1;
        }
    }
    
    int NumSample = counter;
    int *PickedSample = NULL;
    PickedSample = new int[NumSample];
    for (j=0; j<NumSample; j++)
    {
        PickedSample[j] = PickedSampleTmp[j];       
    }
    delete[] PickedSampleTmp;
    
    
    for (int ii=0;ii<m;ii++)
    {
        i = iset[ii] - 1;
        
//         ///////////////////////////////////////
//         // -- Print Screen to Monitor --     //
//         if(i%10==0)
//         {
//             printf("Doing %3d / %3d \n", i, m);
//             mexEvalString("drawnow;");
//         }
//         // -- End Print Screen to Monitor -- //
//         ///////////////////////////////////////

        num = 0.0;
        den = 0.0;
        for (int jj=0;jj<NumSample;jj++)
        {
                j = PickedSample[jj];

                i1 = j % M;
                j1 = (int)(j/M);

                sse = 0.0;
                for (j2=0; j2<winLength; j2++)
                for (i2=0; i2<winLength; i2++)
                {
                    res = y[i*winLength*winLength+j2*winLength+i2]-x[(j1+j2)*Mp+(i1+i2)];
                    sse = sse + res*res;
                }
                w = exp(-sse*invh2);

                num = num + w*x[(j1+win)*Mp+(i1+win)];
                den = den + w;

        }
        znum[ii] = num;
        zden[ii] = den;
        ActualNumSample[ii] = NumSample;
    }
}  


//         for (int j=0;j<n;j++)
//         {
//             if ((double)rand()/RAND_MAX < SamplingRatio)
//             {
//             }
//         }




/////////////////////////
// Interface Function
/////////////////////////
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{

    double *znum;   // znum  is the denoised output
    double *zden;   // zden  is the denoised output
    double *ActualNumSample;
    
    double *y;      // y     is noisy input
    double *x;      // x     is database input
    double *iset;
    
    double h;
    int win;
    
    double SamplingRatio;
            
    int d, n, m;    // n is number of pixels in the database
    int Mp, Np;
    
    const mwSize *xSize;
    mwSize xDims;
    
    const mwSize *ySize;
    mwSize yDims;

    /////////////////////
    // Inputs
    // nlm_external_mean(y,ymean,yquan,x,xmean,xquan,h,tau,jjset);
    /////////////////////
    // Input 0 (noisy patches)
    yDims = mxGetNumberOfDimensions(prhs[0]);
    ySize = mxGetDimensions(prhs[0]);
    y     = mxGetPr(prhs[0]);

    // Input 1 (database patches)
    xDims = mxGetNumberOfDimensions(prhs[1]);
    xSize = mxGetDimensions(prhs[1]);
    x     = mxGetPr(prhs[1]);

    // Input 2 (parameter)
    h = mxGetScalar(prhs[2]);
    if (h<=0)
        mexErrMsgTxt("The parameter hr must be positive. Please retry. \n");

    // Input 3 (parameter)
    win = mxGetScalar(prhs[3]);
    
    // Input 4 (parameter)
    SamplingRatio =  mxGetScalar(prhs[4]);

    // Input 5
    iset = mxGetPr(prhs[5]);
    m    = MAX(mxGetM(prhs[5]), mxGetN(prhs[5]));
    
    // Calculate Output Dimensions
    Mp = xSize[0];
    Np = xSize[1];
   
    d = ySize[0]*ySize[1];
    n = (Mp-2*win)*(Np-2*win);

    int zDims;
    int zSize[2];
    
    zDims    = 2;
    zSize[0] = m;
    zSize[1] = 1;

    ////////////////////
    // Outputs
    ////////////////////
    plhs[0]     = mxCreateNumericArray(zDims, zSize, mxDOUBLE_CLASS, mxREAL);
    znum        = (double *)mxGetData(plhs[0]);  

    plhs[1]     = mxCreateNumericArray(zDims, zSize, mxDOUBLE_CLASS, mxREAL);
    zden        = (double *)mxGetData(plhs[1]);  

    plhs[2]      = mxCreateNumericArray(zDims, zSize, mxDOUBLE_CLASS, mxREAL);
    ActualNumSample = (double *)mxGetData(plhs[2]);  
    
    main_stuff(znum,zden,ActualNumSample,y,x,h,win,SamplingRatio,d,n,m,Mp,Np,iset);
}

