clear all
close all
clc

mSize = matlabpool('size');
if mSize==0
    matlabpool open;
end

addpath(genpath('../offshelf_packages/'));
addpath(genpath('../code/'));
addpath(genpath('../utilities/'));

currentFolder = pwd;

% Parameter
SearchWin = 10;
win       = 2;
npca      = 10;

hs = SearchWin/3;
ws = fspecial('gaussian',[2*SearchWin+1,2*SearchWin+1],hs);
ws = ws(:)/max(ws(:));

nu = 10.^(linspace(-18,3,10000));
phi = zeros(1,length(nu));
for i=1:length(nu)
    phi(i) = sum( max(1e-3, min( 1, abs(ws)/sqrt(nu(i))) ) );
end

% Load data
folder = '../data/standard_images/';
list_fname = dir(folder);


% Sampling Ratio
xi_set = [0.05 0.1 0.2 0.5];


% Noise Variances
sigma_set = [10 20 30 40 50];


for sidx = 3
    sigma = sigma_set(sidx);
    
%     Initialize
    psnr_full = zeros(10,1);
    psnr_ours = zeros(10,5);
    psnr_star = zeros(10,5);
    psnr_kd   = zeros(10,1);
    psnr_am   = zeros(10,1);
    
    for fid=7
        fprintf('fid = %3g \n', fid);
        
        fname = sprintf('%s%s', folder, list_fname(fid+2).name);
        f = im2double(imread(fname));
        if size(f,3)==3
            f = rgb2gray(f);
        end
        
        g = f + random('normal',0, sigma/255, size(f));
        g(g>1) = 1;
        g(g<0) = 0;
        
        hr     = 1.3*sigma/255;
        gpad   = padarray(g,[win,win],'symmetric');
        [M,N]  = size(g);

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Full
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        t0 = tic;
        fhat1   = mcnlm(g,hr,hs,win,SearchWin,1,'uniform');
        tfull   = toc(t0);
        psnr_full(fid) = -10*log10(norm(fhat1(:)-f(:))^2/numel(f));
        fprintf('psnr_full = %3.2f\n', psnr_full(fid));
%         imwrite(fhat1,'man_sigma30_xi100.png');
        

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % KD Tree
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        t0 = tic;
        
        cd ../offshelf_packages/;
        imwrite(g,'noisy.png');
        hr = 1.3*sigma/255;
        npca = 10;
        patch_diameter = 2*win+1;
        s = (patch_diameter - 1)/6;
        full_cmd = sprintf('ImageStack.exe -load noisy.png -nlmeans %3.4g %3.4g %3.4g %3.4g exact -save output.png',s,npca,hs,hr*sqrt(npca));        
        system(full_cmd);
        fhat3 = im2double(imread('output.png'));
        cd(currentFolder);
        
        tkd = toc(t0);
        
        psnr_kd(fid) = -10*log10(norm(fhat3(:)-f(:))^2/numel(f));
        fprintf('psnr kd = %3.2f\n', psnr_kd(fid));
%         imwrite(fhat3,'man_sigma30_gkd.png');
        

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Adaptive Manifold
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        t0 = tic;
        hr = 2*sigma/255;  
        [~,fhat4]          = adaptive_manifold_filter(g, hs, hr); 
        fhat4              = fhat4(:,:,1);
        tam = toc(t0);
        psnr_am(fid)       = -10*log10(norm(fhat4(:)-f(:))^2/numel(f));
        fprintf('psnr am = %3.2f\n', psnr_am(fid));
%         imwrite(fhat4,'man_sigma30_am.png');


        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Monte Carlo
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        hr = 1.3*sigma/255;
        txi = zeros(1,length(xi_set));
        for xi_idx = 1:length(xi_set)
            xi       = xi_set(xi_idx);

            max_trial = 1;
            psnr_tmp2a = zeros(max_trial,1);
            psnr_tmp2b = zeros(max_trial,1);

            t0 = tic;
            for trial = 1:max_trial
                fhat2b = mcnlm(g,hr,hs,win,SearchWin,xi,'uniform');
                psnr_tmp2b(trial) = -10*log10(norm(fhat2b(:)-f(:))^2/numel(f));
            end
            txi(xi_idx) = toc(t0);
            psnr_star(fid,xi_idx) = mean(psnr_tmp2b);
            fprintf('SamplingRatio = %3g, uniform = %3.2f dB\n', xi, mean(psnr_tmp2b));
%             imwrite(fhat2,sname);
        end
    end
end

fprintf('\n\nRuntime: \n');
fprintf('xi = 0.05 \t xi = 0.1 \t xi = 0.2 \t xi = 0.5 \t xi = 1 \t GKD \t AM \n');
fprintf('%3.3f \t & \t %3.3f \t & \t %3.3f \t & \t %3.3f \t & \t %3.3f \t & \t %3.3f \t & \t %3.3f \n', txi(1), txi(2), txi(3), txi(4), tfull, tkd, tam);