%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This routine generates result for Supplementary
% material Figure 2
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
close all
clc

addpath(genpath('../offshelf_packages/'));
addpath(genpath('../code/'));
addpath(genpath('../utilities/'));

currentFolder = pwd;

% Outer Parameters
SearchWin = 10;
win       = 2;
sigma     = 30;
hs = SearchWin/3;

% Load data
f      = im2double(imread('../data/standard_images/man.png'));
g      = f + random('normal',0, sigma/255, size(f));
g(g>1) = 1;
g(g<0) = 0;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Full NLM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
hr        = 1.3*sigma/255;
fhat_full = mcnlm(g,hr,hs,win,SearchWin,1,'uniform');
psnr_full = psnr(fhat_full,f);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% KD Tree
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cd ../offshelf_packages/;
imwrite(g,'noisy.png');
hr = 1.3*sigma/255;
npca = 10;
patch_diameter = 2*win+1;
s = (patch_diameter - 1)/6;
full_cmd = sprintf('ImageStack.exe -load noisy.png -nlmeans %3.4g %3.4g %3.4g %3.4g exact -save output.png',s,npca,hs,hr*sqrt(npca));
system(full_cmd);
fhat_kd = im2double(imread('output.png'));
psnr_kd = psnr(fhat_kd,f);
cd(currentFolder);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Adaptive Manifold
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
hr = 2*sigma/255;
[~,fhat] = adaptive_manifold_filter(g, hs, hr);
fhat_am  = fhat(:,:,1);
psnr_am  = psnr(fhat_am,f);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MCNLM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
hr = 1.3*sigma/255;
fhat_005 = mcnlm(g,hr,hs,win,SearchWin,0.05,'spatial');
psnr_005 = psnr(fhat_005, f);

fhat_01  = mcnlm(g,hr,hs,win,SearchWin,0.1,'spatial');
psnr_01  = psnr(fhat_01, f);



imshow(g); export_fig supp_man_noisy.eps -transparent;
imshow(fhat_full); export_fig supp_man_full.eps -transparent;
imshow(fhat_005); export_fig supp_man_005.eps -transparent;
imshow(fhat_01); export_fig supp_man_01.eps -transparent;
imshow(fhat_kd); export_fig supp_man_kd.eps -transparent;
imshow(fhat_am); export_fig supp_man_am.eps -transparent;
