function kernel = make_kernel_pyramid(win, dim)
% build pyramid kernel
% 
% input: win - radius of the kernel
%        dim - dimension of signal; 1 - time series
%                                   2 - gray scale image
%                                   3 - color image
% output: kernel
%
% SC @ Harvard
% Create:  3 Jun, 2013
% Update: 19 Jul, 2013

if win==0 % When patch size = 1x1
    kernel = 1;
    
else % When patch size > 1x1

    switch dim
        case 1
            kernel=zeros(2*win+1,1);
            for d=1:win
                value= 1 / (2*d+1) ;
                for i=-d:d
                    kernel(win+1-i)= kernel(win+1-i) + value ;
                end
            end
            kernel = kernel/sum(kernel(:));
            
        case 2
            kernel=zeros(2*win+1,2*win+1);
            for d=1:win
                value= 1 / (2*d+1)^2 ;
                for i=-d:d
                    for j=-d:d
                        kernel(win+1-i,win+1-j)= kernel(win+1-i,win+1-j) + value ;
                    end
                end
            end
            kernel = kernel/sum(kernel(:));

        case 3
            kernel=zeros(2*win+1,2*win+1,3);
            for d=1:win
                value= 1 / (2*d+1)^2 ;
                for i=-d:d
                    for j=-d:d
                        kernel(win+1-i,win+1-j,1)= kernel(win+1-i,win+1-j,1) + value ;
                        kernel(win+1-i,win+1-j,2)= kernel(win+1-i,win+1-j,2) + value ;
                        kernel(win+1-i,win+1-j,3)= kernel(win+1-i,win+1-j,3) + value ;
                    end
                end
            end
            kernel = kernel/sum(kernel(:));
            
        otherwise
            error('please enter dim');
    end
end
